'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var defaultComparitor = function defaultComparitor(compareKey) {
  return function (item, newItem) {
    return item[compareKey] === newItem[compareKey];
  };
};

var deepMergeObjects = function deepMergeObjects(objectOld, objectNew) {
  var recurseMerge = function recurseMerge(oldObjectLayer, newObjectLayer) {
    var mergedObject = {};

    for (var key in newObjectLayer) {
      var oldItem = oldObjectLayer[key];
      var newItem = newObjectLayer[key];
      var isMergable = typeof oldItem === 'object' && typeof newItem === 'object' && !Array.isArray(oldItem) && !Array.isArray(newItem);

      if (isMergable) {
        mergedObject[key] = _extends({}, oldObjectLayer[key], recurseMerge(oldItem, newItem));
      } else {
        mergedObject[key] = newItem;
      }
    }

    return mergedObject;
  };

  return _extends({}, objectOld, recurseMerge(objectOld, objectNew));
};
/**
 * Merge two arrays of objects
 * @param items Object array to merge new data into
 * @param newItems Object or array of objects to merge into the first array
 * @param keyOrCallback String or callback to match objects
 * @param mergeDeep On match, merged all nested objects
 */


var mergeBy = function mergeBy(items, newItems, keyOrCallback, mergeDeep) {
  if (mergeDeep === void 0) {
    mergeDeep = false;
  }

  var newItemsArray = !Array.isArray(newItems) ? [newItems] : newItems.filter(function (e) {
    return e;
  });
  var comparitor = typeof keyOrCallback === 'string' ? defaultComparitor(keyOrCallback) : keyOrCallback;
  var mergedItemIndexes = [];
  var mergedItems = items.map(function (item) {
    var matchedIndex = newItemsArray.findIndex(function (newItem) {
      return comparitor(item, newItem);
    });

    if (matchedIndex === -1) {
      return item;
    }

    mergedItemIndexes.push(matchedIndex);

    if (mergeDeep) {
      return deepMergeObjects(item, newItemsArray[matchedIndex]);
    }

    return _extends({}, item, newItemsArray[matchedIndex]);
  }); // Append unmerged items to the array

  var unmergedItems = newItemsArray.filter(function (_, i) {
    return mergedItemIndexes.indexOf(i) === -1;
  });
  return mergedItems.concat(unmergedItems);
};

exports.default = mergeBy;
//# sourceMappingURL=mergeby.cjs.development.js.map
